$(document).ready(function() {
	"use strict"

	var bus = '/hmb/imex';
	var heartbeatInterval = 60;  // seconds
	var clientName = 'evmon';
	var queue = 'SC3MSG'
	var topics = ['EVENT', 'LOCATION', 'MAGNITUDE', 'IMPORT_GROUP'];
	var backfill = 5000;
	var siren = new Audio("sound/siren03.wav");

	var originEvent = {};
	var magnitudeEvent = {};
	var origin = {};
	var magnitude = {};
	var region = {};

	var initfail = false;
	var realtime = false;
	var gotData = false;
	var cfg = null;
	var sid = null;
	var oid = null;

	var errorPopup = $('<div id="errorPopup" title="Error"><div id="errorText" /></div>').dialog({autoOpen: false});

	function pad2(n) {
		return ('0' + n).slice(-2);
	}

	function formatTime(t) {
		return t.getUTCFullYear() + '-' + pad2(t.getUTCMonth()+1) + '-' + pad2(t.getUTCDate()) + ' ' +
			pad2(t.getUTCHours()) + ':' + pad2(t.getUTCMinutes()) + ':' + pad2(t.getUTCSeconds() | 0);
	}

	function addLog(error, msg) {
		var log = $('#log');
		var div = $('<div>' + formatTime(new Date()) + ' ' + msg + '</div>');

		if (error === 'alert') {
			div.css('color', 'blue');
		}
		else if (error) {
			$('#errorText').html(div.clone());
			errorPopup.dialog('open');
			div.css('color', 'red');
		}
		else {
			errorPopup.dialog('close');
		}

		log.append(div);
	}

	function addSystemAlert(level, msg) {
		var closeButton = $('<a class="trac-close-msg" href="#" title="Hide this notice"><span>close</span></a>');
		var msgDiv = $('<div class="system-message ' + level + '">' + msg + '</div>').append(closeButton);

		addLog('alert', msg);

		closeButton.show().click(function () {
			$(this).closest(".system-message").hide();
			return false;
		})

		$('#GEcontent').prepend(msgDiv);
	}

	var evsym = {
		automatic: "<font style='color: red;'>A</font>",
		confirmed: "<font style='color: green;'>C</font>",
		manual: "<font style='color: green;'>M</font>"
	};

	function updateEqtable(e, o, m, r) {
		var eqtable = $('#eqtable');
		var oldtr = $('#' + e.publicID);
		var oldmag = oldtr.children('.mag').text();
		var newtr = $("<tr id='" + e.publicID + "'>" +
			"<td align='left'>" + formatTime(new Date(o.time.value)) + "</td>" +
			"<td class='mag'>" + (m? m.magnitude.value.toFixed(1) : oldmag) + "</td>" +
			"<td align='right'>" + Math.abs(o.latitude.value).toFixed(2) + (o.latitude.value>0 ? " N" : " S") + "</td>" +
			"<td align='right'>" + Math.abs(o.longitude.value).toFixed(2) + (o.longitude.value>0 ? " E" : " W") + "</td>" +
			"<td class='depth'>" + o.depth.value.toFixed(1) + "</td>" +
			"<td align='center'>" + evsym[o.evaluationMode] + "</td>" +
			"<td align='center'>" + (e.preferredFocalMechanismID? "MT" : "") + "</td>" +
			"<td align='left'>" + r + "</td>" +
		"</tr>");

		if (oldtr.length) {
			newtr.attr('class', oldtr.attr('class'));
			oldtr.replaceWith(newtr);
		}
		else {
			newtr.attr('class', (eqtable.children().first().attr('class') == 'oddrow') ? 'evnrow' : 'oddrow');
			eqtable.prepend(newtr);

			if (realtime)
				siren.play();
		}
	}

	function consider(e) {
		var o, m, r;

		if (e && (o = origin[e.preferredOriginID]) &&
				(r = region[e.publicID]) &&
				((m = magnitude[e.preferredMagnitudeID]) || realtime))
			updateEqtable(e, o, m, r);
	}

	function handleEvent(e) {
		originEvent[e.preferredOriginID] = e;
		if (Object.keys(originEvent).length > 1000)
			delete originEvent[Object.keys(originEvent)[0]];

		magnitudeEvent[e.preferredMagnitudeID] = e;
		if (Object.keys(magnitudeEvent).length > 1000)
			delete magnitudeEvent[Object.keys(magnitudeEvent)[0]];

		if (e.description) {
			for (var i in e.description) {
				var desc = e.description[i];
				if (desc.type == "region name") {
					region[e.publicID] = desc.text;
					if (Object.keys(region).length > 1000)
						delete region[Object.keys(region)[0]];
				}
			}
		}

		consider(e);
	}

	function handleOrigin(o) {
		origin[o.publicID] = o;
		if (Object.keys(origin).length > 1000)
			delete region[Object.keys(origin)[0]];

		consider(originEvent[o.publicID]);
	}

	function handleMagnitude(m) {
		magnitude[m.publicID] = m;
		if (Object.keys(magnitude).length > 5000)
			delete region[Object.keys(magnitude)[0]];

		consider(magnitudeEvent[m.publicID]);
	}

	function handleDataMessage(msg) {
		if ('Origin' in msg) {
			for (var i in msg.Origin)
				handleOrigin(msg.Origin[i]);
		}
		else if ('Magnitude' in msg) {
			for (var i in msg.Magnitude)
				handleMagnitude(msg.Magnitude[i]);
		}
		else if ('Event' in msg) {
			for (var i in msg.Event)
				handleEvent(msg.Event[i]);
		}
	}

	function handleNotifierMessage(msg) {
		for (var i in msg.notifier) {
			var notifier = msg.notifier[i];
			if (notifier.operation == 'add' || notifier.operation == 'update') {
				if ('Origin' in notifier)
					handleOrigin(notifier.Origin);
				else if ('Magnitude' in notifier)
					handleMagnitude(notifier.Magnitude);
				else if ('Event' in notifier)
					handleEvent(notifier.Event);
			}
		}
	}

	function longPoll() {
		$.get(bus + '/recv/' + sid + ((oid !== null)? '/' + oid : '')  + '?_=' + $.now(), function(data) {
			var reopen = false;

			try {
				for (var i in data) {
					var obj = data[i];

					if (obj.type == 'SC3') {
						try {
							if ('data_message' in obj.data)
								handleDataMessage(obj.data.data_message);
							else if ('notifier_message' in obj.data)
								handleNotifierMessage(obj.data.notifier_message);
							else if ('IMEXMessage' in obj.data)
								handleNotifierMessage(obj.data.IMEXMessage.notifier_message);
						}
						catch (e) {
							addLog(true, "Page update failed: " + e.message);
						}
					}
					else if (obj.type == 'SYSTEM_ALERT') {
						addSystemAlert(obj.data.level, obj.data.text);
					}
					else if (obj.type == 'NEW_QUEUE') {
						if (obj.data.name == queue || obj.data.name == 'SYSTEM_ALERT')
							reopen = true;
					}
					else if (obj.type == 'HEARTBEAT') {
						realtime = true;
					}

					if (obj.seq != null) {  // not a HEARTBEAT or EOF
						oid = obj.queue + '/' + obj.seq;
						cfg.queue[obj.queue].seq = obj.seq + 1;
					}
				}

				gotData = true;
			}
			catch (e) {
				addLog(true, "Invalid data received: " + e.message);
			}

			if (reopen)
				window.setTimeout(open);
			else
				window.setTimeout(longPoll);

		}).fail(function(jqxhr, statusText) {
			var err = null;

			if (jqxhr.readyState == 4) {
				if (jqxhr.status == 500)
					err = jqxhr.statusText;
				else
					err = jqxhr.responseText;
			}
			else if (statusText != 'error') {
				err = statusText;
			}

			if (err)
				addLog(true, "Error: " + err);

			if (gotData) {
				addLog(false, "Connection to message bus lost");
				window.setTimeout(open);
				gotData = false;
			}
			else {
				addLog(true, "Connection to message bus lost, retrying in 10 seconds");
				window.setTimeout(open, 10000);
			}
		})
	}

	function open() {
		$.post(bus + '/open', JSON.stringify(cfg), function(ack) {
			sid = ack.sid;
			oid = null;

			addLog(false, "Session opened, sid=" + sid);
			addLog(false, "ANNOUNCEMENT: seq=" + ack.queue['ANNOUNCEMENT'].seq);
			addLog(false, "SYSTEM_ALERT: seq=" + ack.queue['SYSTEM_ALERT'].seq);
			addLog(false, queue + ": seq=" + ack.queue[queue].seq);

			window.setTimeout(longPoll);

		}).fail(function(jqxhr, statusText) {
			var err = null;

			if (jqxhr.readyState == 4) {
				if (jqxhr.status == 500)
					err = jqxhr.statusText;
				else
					err = jqxhr.responseText;
			}
			else if (statusText != 'error') {
				err = statusText;
			}

			if (!initfail) {
				if (err)
					addLog(true, "Error: " + err);

				addLog(true, "Connection to message bus failed, retrying in 10 seconds");
				initfail = true;
			}

			window.setTimeout(open, 10000);
		})
	}

	function init() {
		$.ajaxSetup({
			timeout: 2000 * heartbeatInterval,  // 2 times the heartbeat interval
			contentType: 'application/json',
			dataType: 'json'
		});

		cfg = {
			cid: clientName,
			heartbeat: heartbeatInterval,
			queue: {}
		};

		cfg.queue['ANNOUNCEMENT'] = {
			seq: -1
		}

		cfg.queue['SYSTEM_ALERT'] = {
			seq: -2
		}

		cfg.queue[queue] = {
			topics: topics,
			seq: -backfill-1,
			filter: {
				'$or': [
					/* scimport */
					{'data.notifier_message.notifier.Event': {'$exists': true}},
					{'data.notifier_message.notifier.Origin': {'$exists': true}},
					{'data.notifier_message.notifier.Magnitude': {'$exists': true}},
					/* scimex */
					{'data.data_message.Event': {'$exists': true}},
					{'data.IMEXMessage.notifier_message.notifier.Origin': {'$exists': true}},
					{'data.IMEXMessage.notifier_message.notifier.Magnitude': {'$exists': true}}
				]
			}
		};

		addLog(false, "Connecting to message bus and requesting last " + backfill + " messages in queue " + queue + " with topics " + topics.join(', '));

		open();
	}

	init();
});

